module CliApplication
  module Log
    class File < CliApplication::Log::Base
      attr_reader :folder
      attr_reader :filename

      def initialize(app, log, class_folder)
        @app = app
        @log = log
        @class_folder = class_folder
        init_log
      end

      def save_app_start_information
        log(I18n.t('config.log_section.start_header'))
        log(I18n.t('config.log_section.app_name', name: $PROGRAM_NAME))
        log(I18n.t('config.log_section.app_script_name', name: ::File.basename($PROGRAM_NAME)))
        log(I18n.t('config.log_section.app_started_at', time: Time.zone.now))
        log("")
      end

      def save_app_run_information
        log(I18n.t('config.log_section.run_header'))
        log("")
      end

      def save_app_finish_information
        log(I18n.t('config.log_section.finish_header'))
        unless @exception.nil?
          log(@exception.message)
          log(@exception.backtrace.join("\n"))
        end
        log(I18n.t('config.log_section.app_executed_at', time: Time.zone.now, executed_at: @app.executed_at.round(3)))
        log(I18n.t('config.log_section.app_memory', value: StTools::Human.memory))
        log(I18n.t('config.log_section.app_exitcode', exitcode: @app.exitcode, status: ((@app.exitcode == 0 ? 'SUCCESS' : 'FAIL'))))
      end

      def log(message)
        ::File.open(@filename, 'a') {|f| f.write message + "\n" }
        ::File.chmod(0777, @filename)
      end


      private


      def init_log
        init_names
        create_folder
        check_folder_and_file
      end

      # Проверка на доступность папки и файла. Если с этим проблема,
      # то выбрасываем исключение, чтобы уже на этапе отладки было понятно, что
      # при запуске приложения из под cron будут проблемы
      def check_folder_and_file
        if ::File.exists?(@filename)
          delete_file
        else
          check_folder
        end
      end

      # Проверка на возможность дописывать в вфайл из под любого пользователя
      # Если права не XXX, то выбрасываем исключение
      def delete_file
        ::File.delete(@filename)
      end

      def check_folder
        if Dir.exists?(@folder)
          # s = Dir.stat(@filename)
          # puts "Dir >>>>>>>>>>>>>>>>>> #{s.mode}"
        end
      end

      def create_folder
        unless Dir.exists?(@folder)
          Dir.mkdir(@folder, 0777)
        end
      end

      def init_names
        if @log.log_section.location.nil?
          # Локация не задана
          @folder = ::File.join(@class_folder, 'logs')
        else
          # Локация в конфиге задана
          @folder = @log.log_section.location
        end
        @filename = ::File.join(@folder, ::File.basename($PROGRAM_NAME, '.rb')+'.log')
      end

    end
  end
end
